package edu.cmu.cs.able.examples.RMI;
/**
* The <code>GaugeConsumerGUI</code> class implements the GUI of GaugeConsumer.
* @author		Hong Yan
* @author 		Bradley Schmerl
* @version		1.0
* @since		JDK1.3.1
*/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
import edu.cmu.cs.able.gaugeInfrastructure.RMI.*;
import edu.cmu.cs.able.gaugeInfrastructure.RMI.Events.*;

class GaugePage extends JPanel implements ItemListener, ActionListener
{
	private JPanel pnUpper;
	private JPanel pnTop;
	private JPanel pnMid;
	private JComboBox cbProperties;
	private JComboBox cbConfigParams;
	private	JTextField tfProperties;
	private	JTextField tfConfigParams;
	private	JPanel pnControl1;
	private	JButton btRefresh1;
	private	JButton btApply1;
	private	JPanel pnControl2;
	private	JButton btRefresh2;
	private	JButton btApply2;
	private	JTextArea taValue;
	private	JScrollPane spValue;
	private JTabbedPane tpParent;
	
	private String propertyItem;
	private String configItem = "";
	private GaugeID theGaugeID;
	private StringPairVector configParams;
	private RMIGaugeConsumer theGaugeConsumer;
	
	public GaugePage(GaugeID theGaugeID, StringPairVector configParams, RMIGaugeConsumer theGaugeConsumer, JTabbedPane tpParent)
	{
		super(new BorderLayout());
		this.theGaugeID = theGaugeID;
		this.theGaugeConsumer = theGaugeConsumer;
		this.configParams = configParams;
		this.tpParent = tpParent;
		
		pnUpper = new JPanel(new BorderLayout());
		
		pnTop = new JPanel(new BorderLayout());
		pnTop.setBorder(BorderFactory.createCompoundBorder(
						BorderFactory.createTitledBorder("Gauge Properties"),
						BorderFactory.createEmptyBorder()));
		
		cbProperties = new JComboBox();
		cbProperties.addItem("Gauge UID");		
		cbProperties.addItem("Gauge Name");
		cbProperties.addItem("Gauge Type");		
		cbProperties.addItem("Model Name");
		cbProperties.addItem("Model Type");
		propertyItem = cbProperties.getSelectedItem().toString();
		cbProperties.addItemListener(this);
		
		tfProperties = new JTextField(36);
		tfProperties.addActionListener(this);
		displayGFGaugeProperties();
		
		pnControl1 = new JPanel();
		pnControl1.setBorder(BorderFactory.createEtchedBorder());
		btRefresh1 = new JButton("Refresh");
		btRefresh1.addActionListener(this);
		pnControl1.add(btRefresh1);
		btApply1 = new JButton("Delete Gauge!");
		btApply1.addActionListener(this);
		pnControl1.add(btApply1);

		pnTop.add(cbProperties, BorderLayout.WEST);
		pnTop.add(tfProperties, BorderLayout.CENTER);                           
		pnTop.add(pnControl1, BorderLayout.SOUTH);                        

		pnMid = new JPanel(new BorderLayout());
		pnMid.setBorder(BorderFactory.createCompoundBorder(
						BorderFactory.createTitledBorder("Gauge Configuration"),
						BorderFactory.createEmptyBorder()));

		cbConfigParams = new JComboBox();
		if (configParams != null) {
			for (int i = 0; i < configParams.size(); i++) {
				cbConfigParams.addItem(configParams.nameAt(i));
			}
		}
		configItem = cbConfigParams.getSelectedItem().toString();
		cbConfigParams.addActionListener(this);
		cbConfigParams.addItemListener(this);

		tfConfigParams = new JTextField(36);
		tfConfigParams.addActionListener(this);
		displayConfig();

		pnControl2 = new JPanel();
		pnControl2.setBorder(BorderFactory.createEtchedBorder());
		btRefresh2 = new JButton("Refresh");
		btRefresh2.addActionListener(this);
		pnControl2.add(btRefresh2);
		btApply2 = new JButton("Apply");
		btApply2.addActionListener(this);
		pnControl2.add(btApply2);

		pnMid.add(cbConfigParams, BorderLayout.WEST);
		pnMid.add(tfConfigParams, BorderLayout.CENTER);                           
		pnMid.add(pnControl2, BorderLayout.SOUTH);                        
		
		taValue = new JTextArea(10, 36);
		taValue.setLineWrap(true);
		taValue.setBorder(BorderFactory.createEmptyBorder());
		spValue = new JScrollPane(taValue, 
									JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, 
									JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		spValue.setBorder(BorderFactory.createCompoundBorder(
						BorderFactory.createTitledBorder("Gauge Reports"),
						BorderFactory.createEtchedBorder()));

		pnUpper.add(pnTop, BorderLayout.NORTH);
		pnUpper.add(pnMid, BorderLayout.SOUTH);		
		
		add(pnUpper, BorderLayout.NORTH);
		add(spValue, BorderLayout.SOUTH);
	}

	public GaugePage(GaugeID theGaugeID, RMIGaugeConsumer theGaugeConsumer, JTabbedPane tpParent)
	{
		super(new BorderLayout());
		this.theGaugeID = theGaugeID;
		this.theGaugeConsumer = theGaugeConsumer;
		this.configParams = configParams;
		this.tpParent = tpParent;
		
		pnUpper = new JPanel(new BorderLayout());
		
		pnTop = new JPanel(new BorderLayout());
		pnTop.setBorder(BorderFactory.createCompoundBorder(
						BorderFactory.createTitledBorder("Gauge Properties"),
						BorderFactory.createEmptyBorder()));
		
		cbProperties = new JComboBox();
		cbProperties.addItem("Gauge UID");		
		cbProperties.addItem("Gauge Name");
		cbProperties.addItem("Gauge Type");		
		cbProperties.addItem("Model Name");
		cbProperties.addItem("Model Type");
		propertyItem = cbProperties.getSelectedItem().toString();
		cbProperties.addItemListener(this);
		
		tfProperties = new JTextField(36);
		tfProperties.addActionListener(this);
		displayGFGaugeProperties();
		
		pnControl1 = new JPanel();
		pnControl1.setBorder(BorderFactory.createEtchedBorder());
		btRefresh1 = new JButton("Refresh");
		btRefresh1.addActionListener(this);
		pnControl1.add(btRefresh1);
		btApply1 = new JButton("Delete Gauge!");
		btApply1.addActionListener(this);
		pnControl1.add(btApply1);

		pnTop.add(cbProperties, BorderLayout.WEST);
		pnTop.add(tfProperties, BorderLayout.CENTER);                           
		pnTop.add(pnControl1, BorderLayout.SOUTH);                        

		pnMid = new JPanel(new BorderLayout());
		pnMid.setBorder(BorderFactory.createCompoundBorder(
						BorderFactory.createTitledBorder("Gauge Configuration"),
						BorderFactory.createEmptyBorder()));

		cbConfigParams = new JComboBox();

		if (cbConfigParams.getSelectedItem() != null) 
			configItem = cbConfigParams.getSelectedItem().toString();
		cbConfigParams.addActionListener(this);
		cbConfigParams.addItemListener(this);

		tfConfigParams = new JTextField(36);
		tfConfigParams.addActionListener(this);
		displayConfig();

		pnControl2 = new JPanel();
		pnControl2.setBorder(BorderFactory.createEtchedBorder());
		btRefresh2 = new JButton("Refresh");
		btRefresh2.addActionListener(this);
		pnControl2.add(btRefresh2);
		btApply2 = new JButton("Apply");
		btApply2.addActionListener(this);
		pnControl2.add(btApply2);

		pnMid.add(cbConfigParams, BorderLayout.WEST);
		pnMid.add(tfConfigParams, BorderLayout.CENTER);                           
		pnMid.add(pnControl2, BorderLayout.SOUTH);                        
		
		taValue = new JTextArea(10, 36);
		taValue.setLineWrap(true);
		taValue.setBorder(BorderFactory.createEmptyBorder());
		spValue = new JScrollPane(taValue, 
									JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, 
									JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		spValue.setBorder(BorderFactory.createCompoundBorder(
						BorderFactory.createTitledBorder("Gauge Reports"),
						BorderFactory.createEtchedBorder()));

		pnUpper.add(pnTop, BorderLayout.NORTH);
		pnUpper.add(pnMid, BorderLayout.SOUTH);		
		
		add(pnUpper, BorderLayout.NORTH);
		add(spValue, BorderLayout.SOUTH);
	}

	public void itemStateChanged(ItemEvent ie)
	{
		updateGFGaugeProperties();
		updateConfig();
		if (ie.getStateChange() == ItemEvent.SELECTED) {
			if (ie.getSource() == cbProperties) {
				propertyItem = cbProperties.getSelectedItem().toString();					
				displayGFGaugeProperties();
			} else if (ie.getSource() == cbConfigParams) { 
				configItem = cbConfigParams.getSelectedItem().toString();					
				displayConfig();
			}
		}
	}

	public void actionPerformed(ActionEvent ae)
	{	
		try {
			updateGFGaugeProperties();			
			updateConfig();
			if (ae.getSource() == tfProperties || ae.getSource() == tfConfigParams) {
				return;
			} else if (ae.getSource() == btApply2) {
				if (theGaugeConsumer != null) {
					System.out.println("trying to configure gauge..."+theGaugeID.toString());
					theGaugeConsumer.configureGauge(theGaugeID, configParams);
				}
			} else if (ae.getSource() == btApply1) {
				if (theGaugeConsumer != null) {
					System.out.println("trying to delete gauge...");
					theGaugeConsumer.deleteGauge(theGaugeID);
					this.tpParent.remove(this);
					//this.tpParent.removeGaugePane(gfProterties.uid)
				}
			} else if (ae.getSource() == btRefresh1) {
				displayGFGaugeProperties();
			} else if (ae.getSource() == btRefresh2) {
				displayConfig();
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void addReportedValue(String aLine) 
	{
		taValue.append(aLine);
		int pos = taValue.getText().length();
		taValue.setCaretPosition(pos);
		taValue.repaint();
	}
	
	public void changeConfig(StringPairVector config)
	{
		configParams = config;
		System.out.println(configParams.toString());
		displayConfig();
	}

	private void updateGFGaugeProperties()
	{
		if (propertyItem.equals("Gauge UID")) {
			theGaugeID.uid = tfProperties.getText();
		} else if (propertyItem.equals("Gauge Name")) {
			theGaugeID.gaugeName = tfProperties.getText();
		} else if (propertyItem.equals("Gauge Type")) {
			theGaugeID.gaugeType = tfProperties.getText();
		} else if (propertyItem.equals("Model Name")) {
			theGaugeID.modelName = tfProperties.getText();
		} else if (propertyItem.equals("Model Type")) {
			theGaugeID.modelType = tfProperties.getText();
		}
	}		

	private void displayGFGaugeProperties()
	{
		String propertyItem = cbProperties.getSelectedItem().toString();
		if (propertyItem.equals("Gauge UID")) {
			tfProperties.setText(theGaugeID.uid);
		} else if (propertyItem.equals("Gauge Name")) {
			tfProperties.setText(theGaugeID.gaugeName);
		} else if (propertyItem.equals("Gauge Type")) {
			tfProperties.setText(theGaugeID.gaugeType);
		} else if (propertyItem.equals("Model Name")) {
			tfProperties.setText(theGaugeID.modelName);
		} else if (propertyItem.equals("Model Type")) {
			tfProperties.setText(theGaugeID.modelType);
		}
		repaint();		
	}		
	
	private void updateConfig()
	{
		for (int i = 0; i < configParams.size(); i++) {
			if (configParams.nameAt(i).equals(configItem)) {
				configParams.setValue(i, tfConfigParams.getText());
				return;
			}
		}
	}		

	private void displayConfig()
	{
		String configItem = "";
		
		if (cbConfigParams.getItemCount() > 0) {
			configItem = cbConfigParams.getSelectedItem().toString();
			for (int i = 0; i < configParams.size(); i++) {
				if (configParams.nameAt(i).equals(configItem)) {
					tfConfigParams.setText(configParams.valueAt(i));
					return;
				}
			}
		}
		repaint();		
	}		

}	
	
public class GaugeConsumerGUI extends JPanel implements ActionListener, ItemListener, ChangeListener
{  
	public RMIGaugeConsumer theGaugeConsumer;

	private JTextField tfGaugeToBeCreated;
	private JComboBox cbGaugeToBeCreated;
	private JButton btSetupParams;
	private JButton btCreateGauge;
	
	private JTextField tfInterest;
	private JComboBox cbInterest;
	private JTabbedPane tpGauges;
	private JButton btSubscribe;
	private JButton btSetEventPattern;
	private ImageIcon iiGaugeTab;
	
	private GaugeID gfInterest;
	public GaugeID gaugeToBeCreated;
	public StringPairVector gaugeToBeCreatedSetup;
	public GaugeEventType gaugeEvent;
	//private StringPairVector configParams;
	private GaugePage gpCurrent;
	
	private String interestedItem;
	private String gaugeToBeCreatedItem;
	//Constructor takes client core class and registers panel to client.

	public GaugeConsumerGUI(RMIGaugeConsumer theGaugeConsumer)
	{  
		this.theGaugeConsumer = theGaugeConsumer;
		gaugeEvent = new GaugeEventType();
		
		gfInterest = new GaugeID();
		gfInterest.uid = "";
		gfInterest.gaugeName = "";
		gfInterest.gaugeType = "";
		gfInterest.modelName = "";    
		gfInterest.modelType = "";    

		gaugeToBeCreated = new GaugeID();
		gaugeToBeCreated.uid = "";
		gaugeToBeCreated.gaugeName = "RMIBWGaugeName";
		gaugeToBeCreated.gaugeType = "edu.cmu.cs.able.gauges.RMI.RMIBWGauge";
		gaugeToBeCreated.modelName = "RMIBWGaugeModelName";    
		gaugeToBeCreated.modelType = "RMIBWGaugeModelType";    
		
		gaugeToBeCreatedSetup = new StringPairVector();

		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		JPanel pnLevel1 = new JPanel(new BorderLayout());
		
		JPanel pnUpper = new JPanel();
		pnUpper.setBorder(BorderFactory.createCompoundBorder(
						BorderFactory.createTitledBorder("Create Gauge"),
						BorderFactory.createEmptyBorder()));
		cbGaugeToBeCreated = new JComboBox();		
		tfGaugeToBeCreated = new JTextField(24);
		tfGaugeToBeCreated.setToolTipText("Press Enter to confirm change");
		tfGaugeToBeCreated.addActionListener(this);		
		btSetupParams = new JButton("SetupParams");			
		btCreateGauge = new JButton("Create");		
		btSetupParams.addActionListener(this); 
		btCreateGauge.addActionListener(this); 
		pnUpper.add(cbGaugeToBeCreated);

		cbGaugeToBeCreated.addItem("Gauge Type");
		cbGaugeToBeCreated.addItem("Gauge UID");		
		cbGaugeToBeCreated.addItem("Gauge Name");
		cbGaugeToBeCreated.addItem("Model Name");
		cbGaugeToBeCreated.addItem("Model Type");
		gaugeToBeCreatedItem = cbGaugeToBeCreated.getSelectedItem().toString();
		
		cbGaugeToBeCreated.addItemListener(this);
		pnUpper.add(tfGaugeToBeCreated);
		pnUpper.add(btSetupParams);
		pnUpper.add(btCreateGauge);
		pnLevel1.add(pnUpper, BorderLayout.NORTH);
		
		JPanel pnTop = new JPanel();
		pnTop.setBorder(BorderFactory.createCompoundBorder(
						BorderFactory.createTitledBorder("Interest of GaugeConsumer"),
						BorderFactory.createEmptyBorder()));
		cbInterest = new JComboBox();		
		tfInterest = new JTextField(24);
		tfInterest.setToolTipText("Press Enter to confirm change");
		tfInterest.addActionListener(this);		
		btSetEventPattern = new JButton("EventPattern");			
		btSubscribe = new JButton("Subscribe");		
		btSubscribe.addActionListener(this); 
		btSetEventPattern.addActionListener(this); 
		pnTop.add(cbInterest);

		cbInterest.addItem("Gauge Type");
		cbInterest.addItem("Gauge UID");		
		cbInterest.addItem("Gauge Name");
		cbInterest.addItem("Model Name");
		cbInterest.addItem("Model Type");
		interestedItem = cbInterest.getSelectedItem().toString();
		
		cbInterest.addItemListener(this);
		pnTop.add(tfInterest);
		pnTop.add(btSetEventPattern);
		pnTop.add(btSubscribe);
		pnLevel1.add(pnTop, BorderLayout.SOUTH);
		
		add(pnLevel1, BorderLayout.NORTH);
		
		JPanel pnBottom = new JPanel(new BorderLayout());
		pnBottom.setBorder(BorderFactory.createCompoundBorder(
						BorderFactory.createTitledBorder("Interested Gauges"),
						BorderFactory.createEmptyBorder()));

        tpGauges = new JTabbedPane();
        tpGauges.addChangeListener(this);
		iiGaugeTab = new ImageIcon("images/gauge48.gif");
		//createGaugePane("test", "test");
		//createGaugePane("test2", "test2");
		pnBottom.add(tpGauges, BorderLayout.CENTER);
		add(pnBottom, BorderLayout.CENTER);
	}                                                        
	
    protected void createGaugePane(GaugeID theGaugeID, StringPairVector configParams, RMIGaugeConsumer theGaugeConsumer)
	{
		for (int i = 0; i < tpGauges.getTabCount(); i++) {
			if (tpGauges.getToolTipTextAt(i).equals(theGaugeID.uid))
				return;
		}
		GaugePage theGaugePage = new GaugePage(theGaugeID, configParams, theGaugeConsumer, tpGauges);
		tpGauges.addTab(theGaugeID.gaugeName, iiGaugeTab, theGaugePage, theGaugeID.uid);
		tpGauges.setSelectedComponent(theGaugePage);
    }

    protected void createGaugePane(GaugeID theGaugeID, RMIGaugeConsumer theGaugeConsumer)
	{
		for (int i = 0; i < tpGauges.getTabCount(); i++) {
			if (tpGauges.getToolTipTextAt(i).equals(theGaugeID.uid))
				return;
		}
		GaugePage theGaugePage = new GaugePage(theGaugeID, theGaugeConsumer, tpGauges);
		tpGauges.addTab(theGaugeID.gaugeName, iiGaugeTab, theGaugePage, theGaugeID.uid);
		tpGauges.setSelectedComponent(theGaugePage);
    }

	public void removeGaugePane(String uid)
	{
		for (int i = 0; i < tpGauges.getTabCount(); i++) {
			if (tpGauges.getToolTipTextAt(i).equals(uid)) {
				tpGauges.remove(i);			
				return;
			}
		}
    }
	
	public void stateChanged(ChangeEvent e)
	{
		gpCurrent = (GaugePage)tpGauges.getSelectedComponent();
	}

	private void updateGaugeToBeCreated()
	{
		String s;
		if (gaugeToBeCreatedItem.equals("Gauge UID")) {
			s = tfGaugeToBeCreated.getText();
			if (!s.equals("")) {
				gaugeToBeCreated.uid = s;
			}
		} else if (gaugeToBeCreatedItem.equals("Gauge Name")) {
			s = tfGaugeToBeCreated.getText();
			if (!s.equals("")) {
				gaugeToBeCreated.gaugeName = s;
			}
		} else if (gaugeToBeCreatedItem.equals("Gauge Type")) {
			s = tfGaugeToBeCreated.getText();
			if (!s.equals("")) {
				gaugeToBeCreated.gaugeType = s;
			}
		} else if (gaugeToBeCreatedItem.equals("Model Name")) {
			s = tfGaugeToBeCreated.getText();
			if (!s.equals("")) {
				gaugeToBeCreated.modelName = s;
			}
		} else if (gaugeToBeCreatedItem.equals("Model Type")) {
			s = tfGaugeToBeCreated.getText();
			if (!s.equals("")) {
				gaugeToBeCreated.modelType = s;
			}
		}
	}		
	
	private void updateGFGaugeInterest()
	{
		if (interestedItem.equals("Gauge UID")) {
			gfInterest.uid = tfInterest.getText();
		} else if (interestedItem.equals("Gauge Name")) {
			gfInterest.gaugeName = tfInterest.getText();
		} else if (interestedItem.equals("Gauge Type")) {
			gfInterest.gaugeType = tfInterest.getText();
		} else if (interestedItem.equals("Model Name")) {
			gfInterest.modelName = tfInterest.getText();
		} else if (interestedItem.equals("Model Type")) {
			gfInterest.modelType = tfInterest.getText();
		}
	}		

	private void displayGFGaugeInterest()
	{
		String interestedItem = cbInterest.getSelectedItem().toString();
		if (interestedItem.equals("Gauge UID")) {
			tfInterest.setText(gfInterest.uid);
		} else if (interestedItem.equals("Gauge Name")) {
			tfInterest.setText(gfInterest.gaugeName);
		} else if (interestedItem.equals("Gauge Type")) {
			tfInterest.setText(gfInterest.gaugeType);
		} else if (interestedItem.equals("Model Name")) {
			tfInterest.setText(gfInterest.modelName);
		} else if (interestedItem.equals("Model Type")) {
			tfInterest.setText(gfInterest.modelType);
		}
		repaint();		
	}		

	private void displayGaugeToBeCreated()
	{
		String gaugeToBeCreatedItem = cbGaugeToBeCreated.getSelectedItem().toString();
		if (gaugeToBeCreatedItem.equals("Gauge UID")) {
			tfGaugeToBeCreated.setText(gaugeToBeCreated.uid);
		} else if (gaugeToBeCreatedItem.equals("Gauge Name")) {
			tfGaugeToBeCreated.setText(gaugeToBeCreated.gaugeName);
		} else if (gaugeToBeCreatedItem.equals("Gauge Type")) {
			tfGaugeToBeCreated.setText(gaugeToBeCreated.gaugeType);
		} else if (gaugeToBeCreatedItem.equals("Model Name")) {
			tfGaugeToBeCreated.setText(gaugeToBeCreated.modelName);
		} else if (gaugeToBeCreatedItem.equals("Model Type")) {
			tfGaugeToBeCreated.setText(gaugeToBeCreated.modelType);
		}
		repaint();		
	}		

	
	// Triggered when a certain interest item is chosend
	public void itemStateChanged(ItemEvent ie)
	{
		updateGFGaugeInterest();
		updateGaugeToBeCreated();
		if (ie.getSource() == cbInterest) {
			if (ie.getStateChange() == ItemEvent.SELECTED) {
				interestedItem = cbInterest.getSelectedItem().toString();
				displayGFGaugeInterest();
			}
		}
		else if (ie.getSource() == cbGaugeToBeCreated) {
			if (ie.getStateChange() == ItemEvent.SELECTED) {
				gaugeToBeCreatedItem = cbGaugeToBeCreated.getSelectedItem().toString();
				displayGaugeToBeCreated();
			}
		}
	}
	
	public void actionPerformed(ActionEvent ae)
	{	
		try {
			updateGFGaugeInterest();		
			updateGaugeToBeCreated();	
			/*if (ae.getSource() == tfGaugeToBeCreated) {
				updateGaugeToBeCreated();
				return;
			} else if (ae.getSource() == tfInterest) {
				updateGFGaugeInterest();
				return;
			} else */
			if (ae.getSource() == btSetEventPattern) {
				if (theGaugeConsumer != null) {
					SubscriptionOptionsDialog subsDialog = new SubscriptionOptionsDialog (this, true, this);
					subsDialog.setModal(true);
					subsDialog.setVisible (true);
				}
			} else if (ae.getSource() == btSubscribe) {
				if (theGaugeConsumer != null) {
					System.out.println(gaugeEvent.toString());
					System.out.println(gfInterest.toString());
					updateGFGaugeInterest();
					theGaugeConsumer.subscribeInterest(gfInterest, gaugeEvent);
				}
			} else if (ae.getSource() == btCreateGauge) {
				if (theGaugeConsumer != null) {
					updateGaugeToBeCreated();
					System.out.println(gaugeToBeCreatedSetup.toString());
					theGaugeConsumer.createGauge(gaugeToBeCreated, gaugeToBeCreatedSetup);
				}
			} else if (ae.getSource() == btSetupParams) {
				if (theGaugeConsumer != null) {
					GaugeCreationDialog setupDialog = new GaugeCreationDialog (gaugeToBeCreated, this);
					setupDialog.setModal(true);
					setupDialog.setVisible (true);
					//theGaugeConsumer.subscribeInterest(gfInterest);
				}
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

    /**
    * Adds a reported value in the GUI. 
    * @param   uid					the Unique ID of the gauge to be configured.    
    * @param   aline					reported value.    
    */
	public void addReportedValue(String uid, String aLine) 
	{
		String tip;
		GaugePage gp;
		for (int i = 0; i < tpGauges.getTabCount(); i++) {
			tip = tpGauges.getToolTipTextAt(i);
			System.out.println(tip);
			if (tip.equals(uid)) {
				gp = (GaugePage)tpGauges.getComponentAt(i);
				gp.addReportedValue(aLine+"\n");
				return;
			}
		}
	}

    /**
    * Changes the configuration of a gauge in the GUI. 
    * @param   uid					the Unique ID of the gauge to be configured.    
    * @param   configParams				the configuration parameters of the gauge to be configured.    
    */
	public void changeConfig(String uid, StringPairVector config)
	{
		String tip;
		GaugePage gp;
		for (int i = 0; i < tpGauges.getTabCount(); i++) {
			tip = tpGauges.getToolTipTextAt(i);
			System.out.println(tip);
			if (tip.equals(uid)) {
				gp = (GaugePage)tpGauges.getComponentAt(i);
				gp.changeConfig(config);
				return;
			}
		}
	}
	
    /**
    * Adds a gauge in the GUI. 
    * @param   theGaugeID		   the filter of the gauge to be added.    
    * @param   configParams			   the configuration parameters of the gauge to be added.    
    */
	public void addGauge(GaugeID theGaugeID, StringPairVector configParams) 
	{
		System.out.println(configParams.toString());
		createGaugePane(theGaugeID, configParams, theGaugeConsumer);
		repaint();
	}

    /**
    * Adds a gauge in the GUI. 
    * @param   theGaugeID		   the filter of the gauge to be added.    
    * @param   configParams			   the configuration parameters of the gauge to be added.    
    */
	public void addGauge(GaugeID theGaugeID) 
	{
		createGaugePane(theGaugeID, theGaugeConsumer);
		repaint();
	}

    /**
    * Deletes a gauge in the GUI. 
    * @param   uid		   the Unique ID of the gauge to be deleted.    
    */
	public void deleteGauge(String uid) 
	{
		System.out.println("trying to delete "+uid);
		removeGaugePane(uid);
		repaint();
	}

	
    /**
    * Displays the GUI. 
    */
	public void display()
	{
		JFrame frame = new JFrame("GaugeConsumerGUI");
		frame.setSize(650, 650);
		/*frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {System.exit(0);}
		});*/
		frame.getContentPane().add("Center", this);
		//frame.pack();
		frame.setVisible(true);
		displayGaugeToBeCreated();
	}	
}
