/*
 * This class implements a RMI bandwidth gauge.
 *
 */

package edu.cmu.cs.able.gauges.RMI;
import java.rmi.*;
import java.util.*;

import java.net.*;
import java.io.*;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.RMI.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;

public class RMITimeGauge extends RMIGauge {

    Date date = null;
    static int frequency = 500;
    static boolean doReporting = false;

    /** Creates new RMITimeGauge
     * @param gaugeID The ID of the gauge to create.
     * @param creatorID The ID of the gauge manager that is creating the gauge.
     * @param setupParams The value of the setup params.
     */
	public RMITimeGauge (GaugeID gaugeID, String creatorID, StringPairVector setupParams)
	{
		super(gaugeID, creatorID, setupParams);
		System.out.println (gaugeID.toString () + " created");
		boolean freq_defined = false;
		boolean report_defined = false;
		
        if (setupParams != null) {
        	for (int i = 0; i < setupParams.size (); i++) {
            	if (setupParams.nameAt (i).equals ("frequency")) {
	                frequency = Integer.parseInt (setupParams.valueAt (i));
	                freq_defined = true;
    	        }
        	    else if (setupParams.nameAt (i).equals ("report")) {
            	    if (setupParams.valueAt (i).equals ("true")) {
                	    doReporting = true;
                	    report_defined = true;
                	}
            	}
        	}
	        System.out.println ("with");
    	    for (int i = 0; i < setupParams.size (); i ++) {
        	    System.out.println ("  setup " + setupParams.nameAt (i) + " = " + setupParams.valueAt (i));
	        }
        	
        }
        if (!freq_defined) {
        	configParams.addElement ("frequency", new Integer (frequency).toString ());
        }
        if (!report_defined) {
        	configParams.addElement ("report", new Boolean (doReporting).toString ());
        }
	}

    public void reportNewValue (){
        //gaugeBus.reportValue(gaugeID, "Bandwidth", "", "10");
        date = new Date ();
        gaugeBus.reportValue (gaugeID, "Date: ", "", date.toString ());
    }

    /** Returns the state of the gauge.
     * @param setupParams The vector into which the setup parameters are copied. Will be
     * empty for this gauge.
     * @param configParams The configuration parameters for this gauge. Will have
     * an element for <pre>frequency</pre> and <pre>report</pre>.
     * @param mappings The property that each gauge value is associated with.
     * Will have an element for <pre>date</pre>.
     * @return Whether the state of the gauge could be queried.
     */    
    public boolean queryState(StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        this.setupParams.copyInto (setupParams);
        configParams.addElement ("frequency", new Integer (frequency).toString ());
        configParams.addElement ("report", new Boolean (doReporting).toString ());
        
        return true;
    }

    /** This method is used to configure the gauge.
     * @param configParams The parameters used to configure the gauge. For the
     * SienaTimeGauge, the valid configuration parameters are:<br>
     * &nbsp;&nbsp;frequency : int (millisecond period of reporting)<br>
     * &nbsp;&nbsp;report : true (whether the gauge should report a value)
     * @return Whether the gauge was successfully configured.
      */    
    public boolean configure(StringPairVector configParams) {
        for (int i = 0; i < configParams.size (); i++) {
            if (configParams.nameAt (i).equals ("frequency")) {
                frequency = Integer.parseInt (configParams.valueAt (i));
            }
            else if (configParams.nameAt (i).equals ("report")) {
                if (configParams.valueAt (i).equals ("true"))
                    doReporting = true;
                else doReporting = false;
            }
        }
        return true;
    }

    /** A command-line interface to start the gauge.
     * @param args -gt <i>gauge type</i><br>
     * -gn <i>gauge name</i><br>
     * -mt <i>model type</i><br>
     * -mn <i>model name</i><br>
     * [-setup <i>name=value</i>]<br>
     * [-mappings <i>name=property</i>]<br>
     * -creator <i>creator ID</i><br>
     */    
	public static void main(String args[])
	{

		String gaugeName = "RMITimeGaugeName";   
		String gaugeType = "edu.cmu.cs.able.gauges.RMI.RMITimeGauge";   
		String modelName = "RMITimeGaugeModelName";   
		String modelType = "RMITimeGaugeModelType";   				    
        String creatorID = "";

        StringPairVector mappings = new StringPairVector ();
        StringPairVector setupParams = new StringPairVector ();
        boolean gotMappings = false;
        boolean gotSetup = false;
        // Process args
        boolean ok = true;
        for (int i = 0; i < args.length && ok; i++) {
            String tag = args [i];
            if (args [i].equals ("-creator")) {
                creatorID = args [++i];
            }
            else if (args [i].equals ("-gn")) {
                gaugeName = args [++i];
            }
            else if (args [i].equals ("-gt")) {
                gaugeType = args [++i];
            }
            else if (args [i].equals ("-mn")) {
                modelName = args [++i];
            }
            else if (args [i].equals ("-mt")) {
                modelType = args [++i];
            }
            else if (args [i].equals ("-setup")) {
                gotSetup = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String setup = args [i];
                    String name = setup.substring (0, setup.indexOf ('='));
                    String value = setup.substring (setup.indexOf ('=') + 1);
                    setupParams.addElement (name, value);
                    i++;
                } 
                i--;
            }
            else if (args [i].equals ("-mappings")) {
                gotMappings = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String mapping = args [i];
                    String value = mapping.substring (0, mapping.indexOf ('='));
                    String property = mapping.substring (mapping.indexOf ('=') + 1);
                    mappings.addElement (value, property);
                    i++;
                }
                i--;
            }
            else if (args [i].equals ("-rmib")) {
            	++i;
            }
            else {
                ok = false;
            }            
        }
        
        if (!ok) {
            printUsageMessage ();
            System.exit (0);
        }
        
        GaugeID gid = new GaugeID ();
        gid.modelName = modelName;
        gid.modelType = modelType;
        gid.gaugeName = gaugeName;
        gid.gaugeType = gaugeType;

   		System.out.println("RMITimeGauge started");
   		System.out.flush();

   		RMITimeGauge theTimeGauge = new RMITimeGauge(gid, creatorID, setupParams);
   		
   		while (true) {
   			if (doReporting) theTimeGauge.reportNewValue();
   			//System.err.println("reported");
   			try {
   				Thread.sleep(frequency);
   			}
   			catch (Exception e) 
   			{
   				e.printStackTrace();
   			}
   		}
	}
    
    private static void printUsageMessage () {
        System.out.println ("Usage: RMITimeGauge -creator <creatorID> -mt <modelType> -mn <modelName> -gt <gaugeType> -gn <gaugeName> -setup <name=value> -mappings <value=property> -rmib <rmibus>");
    }
    
}
