/*
 * BandwidthGauge.java
 *
 * Created on November 20, 2001, 9:15 AM
 */

package edu.cmu.cs.able.gauges.Siena.Demo;

import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.probebus.*;
import edu.cmu.cs.able.probebus.siena.*;
import edu.cmu.cs.able.probebus.events.*;
import edu.cmu.cs.able.probebus.siena.events.*;
/** Implementation of the BandwidthGauge, conforming to gauge type:<br>
 * ServerLoadGauge = [<br>
 * &nbsp;&nbsp;type = edu.cmu.cs.able.gauges.Siena.Demo.BandwidthGauge<br>
 * &nbsp;&nbsp;setup = [where : String]<br>
 * &nbsp;&nbsp;config = []<br>
 * &nbsp;&nbsp;values = [bandwidth : float]
 * @author Bradley Schmerl
 * @version 1.0
 */
public class BandwidthGauge extends edu.cmu.cs.able.gaugeInfrastructure.Siena.SienaGauge implements IProbeConsumer{
    /** Where (the component or connector) the gauge is attached. (maps to the probe shell name)
     */    
    protected String where = null;
    private static final Double ZERO = new Double (0.0);
    public static final double ALPHA = 0.5;
    /** The value reported by the gauge
     */
    Double bandwidth = ZERO;
    Double cumulative = ZERO;
    int number = 0;
    String bwMapping = "";
    
    public String gaugeType = "edu.cmu.cs.able.gauges.Siena.Demo.BandwidthGauge";
    
    private static IProbeBus probeBus = null;
    private final String CONFIG_NAME = "BWConfig";
    private final String TARGET_CLASSES = "runtime.client.Client,runtime.client.ClientThread";
    private KXProbeHelper bwProbe;
    private IProbeEventFactory probeEventFactory = null;
    /** Creates new BandwidthGauge     
     * @param gaugeID The ID of the new gauge.
     * @param creatorID The ID of the gauge manager that created the gauge.
     * @param setupParams The parameters that the gauge uses to set itself up. The
     * valid setup parameters for this gauge are:<br>
     * &nbsp;&nbsp;where : String
     * @param mappings The property that the <pre>latency</pre> value is associated with.
     */
    public BandwidthGauge (GaugeID gaugeID, String creatorID, StringPairVector setupParams, StringPairVector mappings) {
        super (gaugeID, creatorID, setupParams, mappings);
        
        for (int i = 0; i < setupParams.size (); i++) {
            if (setupParams.nameAt (i).equals ("where")) {
                where = setupParams.valueAt (i);
            }
        }
 
        bwMapping = (String )this.mappings.get ("bandwidth");
        probeEventFactory = new SienaProbeEventFactory( (SienaProbeBus )probeBus );
        
        sleepTime = 3000;
        doReporting = false; // Gauge will report when it receives an event from a probe, so
        // this is false - should be renamed to doPeriodicReporting
        bwProbe = new KXProbeHelper (CONFIG_NAME, where, TARGET_CLASSES, this, probeBus);           
    }

    private void probesDeployed () {
        finalizeCreation (myManagerID, true);
        IProbeEvent e = probeEventFactory.createBandwidthEvent( "", CONFIG_NAME, where ); 
        probeBus.registerForEvent (e, this);
    }
    
    /** This method is used to configure the gauge.
     * @param configParams The parameters used to configure the gauge. For the
     * BandwidthGauge, there are no valid configuration parameters for this gauge.
     * @return Whether the gauge was successfully configured. Will always return false.
      */    
    public boolean configure (StringPairVector configParams) {
        return false;
    }
    
    /** Returns the current date.
     * @param values The vector into which to place the date.
     * @return Whether the gauge could report it's current value.
     */    
    public boolean queryAllValues (GaugeValueVector values) {
        synchronized (cumulative) {
            Double bandwidth = new Double (cumulative.doubleValue ()/number);
            values.addElement ("bandwidth", (String )mappings.get ("bandwidth"), bandwidth.toString ());
        }
        return true;
    }
    
    /** Returns the state of the gauge.
     * @param setupParams The vector into which the setup parameters are copied. Will be
     * empty for this gauge.
     * @param configParams The configuration parameters for this gauge. Will be empty.
     * @param mappings The property that each gauge value is associated with.
     * Will have an element for <pre>bandwidth</pre>.
     * @return Whether the state of the gauge could be queried.
     */    
    public boolean queryState (StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
        this.setupParams.copyInto (setupParams);
        configParams.removeAllElements ();
        
        java.util.Enumeration enum = this.mappings.keys ();
        while (enum.hasMoreElements ()) {
            String key = (String )enum.nextElement ();
            mappings.addElement (key, (String )this.mappings.get (key));
        }
        
        return true;
    }
    
    /** The method called to query the value of the gauge. This
     * provides a direct query for the gauge, as opposed to the
     * gauge reporting to value on the bus.
     * @param valueName The name of the value to query.
     * @return The corresponding value of the gauge. (Note: <I>it is assumed
     * that the querier know the property associated with this
     * value</I>).
     */    
    public String queryValue(String valueName) {
        if (valueName.equals ("bandwidth")) {
            String bw = "";
            synchronized (cumulative) {
                bw = Double.toString (cumulative.doubleValue ()/number);
            }
            return bw;
        }
        else {
            return "";
        }
    }
    
    /** A command-line interface to start the gauge.
     * @param args -gt <i>gauge type</i><br>
     * -gn <i>gauge name</i><br>
     * -mt <i>model type</i><br>
     * -mn <i>model name</i><br>
     * [-setup <i>name=value</i>]<br>
     * [-mappings <i>name=property</i>]<br>
     * -creator <i>creator ID</i><br>
     * -senp <i>siena port</i><br>
     * [-debug]
     */    
    public static void main (String [] args) {

        
        String creatorID = null;
        String gaugeName = null;
        String gaugeType = null;
        String modelName = null;
        String modelType = null;
        StringPairVector mappings = new StringPairVector ();
        StringPairVector setupParams = new StringPairVector ();
        boolean gotMappings = false;
        boolean gotSetup = false;
        // Process args
        boolean ok = true;
        for (int i = 0; i < args.length && ok; i++) {
            String tag = args [i];
            if (args [i].equals ("-creator")) {
                creatorID = args [++i];
            }
            else if (args [i].equals ("-gn")) {
                gaugeName = args [++i];
            }
            else if (args [i].equals ("-gt")) {
                gaugeType = args [++i];
            }
            else if (args [i].equals ("-mn")) {
                modelName = args [++i];
            }
            else if (args [i].equals ("-mt")) {
                modelType = args [++i];
            }
            else if (args [i].equals ("-setup")) {
                gotSetup = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String setup = args [i];
                    String name = setup.substring (0, setup.indexOf ('='));
                    String value = setup.substring (setup.indexOf ('=') + 1);
                    setupParams.addElement (name, value);
                    i++;
                } 
                i--;
            }
            else if (args [i].equals ("-mappings")) {
                gotMappings = true;
                i++;
                while (i < args.length && args [i].charAt (0) != '-') {
                    String mapping = args [i];
                    String value = mapping.substring (0, mapping.indexOf ('='));
                    String property = mapping.substring (mapping.indexOf ('=') + 1);
                    mappings.addElement (value, property);
                    i++;
                }
                i--;
            }
            else if (args [i].equals ("-gb")) {
                String senp = args [++i];
                edu.cmu.cs.able.gaugeInfrastructure.Siena.Initialization.initSiena (senp);
//                probeBus = new SienaProbeBus ("senp://snowgum.wv.cs.cmu.edu:7130");
            }
            else if (args [i].equals ("-pb")) {
                probeBus = new SienaProbeBus (args [++i]);
            }
            else if (args [i].equals ("-debug")) {
                edu.cmu.cs.able.gaugeInfrastructure.util.Global.debugFlag = true;
            }
            else {
                ok = false;
            }            
        }
        
        if (!ok) {
            printUsageMessage ();
            System.exit (0);
        }
        
        GaugeID gid = new GaugeID ();
        gid.modelName = modelName;
        gid.modelType = modelType;
        gid.gaugeName = gaugeName;
        gid.gaugeType = gaugeType;
   
       
        BandwidthGauge gauge = new BandwidthGauge (gid, creatorID, setupParams, mappings);
       
        
    }
    
    private static void printUsageMessage () {
        System.out.println ("Usage: BandwidthGauge -creator <creatorID> -mt <modelType> -mn <modelName> -gt <gaugeType> -gn <gaugeName> -setup <name=value> -mappings <value=property> -senp <sienaport>");
        
    }
    
    /** The callback invoked when a probe reports a single event.
     * @param e.  The event that was reported.
     */
    public void onProbeEvent (IProbeEvent event) {
        if (event instanceof SienaActivateResponseProbeEvent) {
            Global.debug ("Got activate response ... finishing creation");
            probesDeployed ();
        }
        else if (event instanceof SienaBandwidthProbeEvent) {
            SienaBandwidthProbeEvent bwEvent =
            (SienaBandwidthProbeEvent )event;
            Global.debug ("Got probe event " + bwEvent.toString ());
            double newBandwidth = new Double (bwEvent.getProbeEvent ().getBandwidth ()).doubleValue ();
            synchronized (cumulative) {
                number++;
                cumulative = new Double (cumulative.doubleValue () + newBandwidth);
            }
            doReporting = true;
        }
    }
    
     public void processDelete (String gaugeMgrID) {
        bwProbe.manageRemoval ();
        finalizeDeletion (gaugeMgrID);
        bwProbe.shutdown ();
        System.out.println("Gauge: " + getGaugeID ().gaugeName + " exiting");
        try {
            Thread.sleep (2000); // some time to wait for things to calm down
        }
        catch (Exception e) {}
        System.exit (0);
    }
    
    /** Reports a new value. This should be overridden by extenders of this gauge.
     */
    public void reportNewValue () {
        Double lBandwidth = null;
        synchronized (cumulative) {
            if (number > 0) {
                lBandwidth = new Double (cumulative.doubleValue ()/number);
                number = 0;
                cumulative = ZERO;
            }
        }
        gaugeBus.reportValue (getGaugeID (), "bandwidth", bwMapping, lBandwidth.toString ());
    }
    
}
