/*
 * KXProbeHelper.java
 *
 * Created on November 27, 2001, 12:10 PM
 */

package edu.cmu.cs.able.gauges.Siena.Demo;
import java.util.Enumeration;
import edu.cmu.cs.able.gaugeInfrastructure.Siena.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.probebus.*;
import edu.cmu.cs.able.probebus.siena.*;
import edu.cmu.cs.able.probebus.events.*;
import edu.cmu.cs.able.probebus.siena.events.*;
import siena.Notification;
import siena.SienaException;
/**
 * Generic helper for KX compatible probes
 * @author  schmerl
 * @version
 */
public class KXProbeHelper implements edu.cmu.cs.able.probebus.IProbeConsumer {
    protected String configName;
    protected String systemID;
    protected String targets;
    protected IProbeConsumer reportSubscriber;
    public static final int UNDEPLOYED = 0;
    public static final int DEPLOYED = 1;
    public static final int INSTALLED = 2;
    public static final int ACTIVE = 3;
    private int depState = 3;
    private IProbeEventFactory probeEventFactory = null;
    private IProbeBus probeBus = null;
    
    public void shutdown () {
        SienaProbeBus pb = (SienaProbeBus )probeBus;
        pb.siena.shutdown ();
    }
    
    /** Creates new KXProbeHelper */
    public KXProbeHelper (String cn, String sid, String cl, IProbeConsumer rs, IProbeBus pb) {
        configName = cn;
        systemID = sid;
        targets = cl;
        reportSubscriber = rs;
        probeBus = pb;
        probeEventFactory = new SienaProbeEventFactory( (SienaProbeBus )probeBus );
        
        findProbeStatus ();
    }
    
    public void setDeployeState (int dState) {
        depState = dState;
    }
    
    private void findProbeStatus () {
        Global.debug ("Finding probe status");
        IProbeEvent event = probeEventFactory.createStatusEvent (systemID);
        IProbeEvent response = probeEventFactory.createStatusResponseEvent (systemID, "", "", "");
        probeBus.registerForEvent (response, this);
        boolean success = event.dispatch ();
    }
    
    private void foundProbeStatus (SienaStatusResponseProbeEvent event) {
        Global.debug ("Found probe status");
        
        boolean deployed = isDeployed (event, configName);
        boolean activated = isActivated (event, configName);
        boolean installed = isInstalled (event, configName);
        
        if (activated && depState == ACTIVE) {
            Global.debug ("Probe is installed...activating");
            activateProbe ();
        }
        else if (deployed && depState >= INSTALLED) {
            Global.debug ("Probe is deployed...installing");
            installProbe ();
        }
        else {
            Global.debug ("Deploying probe");
            deployProbe ();
        }
    }
    
    public void activateProbe () {
        if (depState >= ACTIVE) {
            IProbeEvent event = probeEventFactory.createActivateEvent (configName, systemID );
            IProbeEvent response = probeEventFactory.createActivateResponseEvent (configName, systemID, "", "");
            probeBus.registerForEvent (response, this);
            boolean success = event.dispatch();      
        }
    }
    
    private void probeActivated (SienaActivateResponseProbeEvent event) {
        reportSubscriber.onProbeEvent (event);
    }
    
    public void installProbe () {
        if (depState >= INSTALLED) {
            IProbeEvent event = probeEventFactory.createInstallEvent ( configName, systemID );
            IProbeEvent response = probeEventFactory.createInstallResponseEvent (configName, systemID, "", "");
            probeBus.registerForEvent (response, this);
            boolean success = event.dispatch();        
        }
    }
    
    private void probeInstalled () {
        activateProbe ();
    }
    
    public void deployProbe () {
        IProbeEvent event = probeEventFactory.createDeployEvent (configName, systemID, targets );
        IProbeEvent response = probeEventFactory.createDeployResponseEvent (configName, systemID, "", "");
        probeBus.registerForEvent (response, this);
        boolean success = event.dispatch();
    }
    
    private void probeDeployed () {
        installProbe ();
    }
    
    private boolean isDeployed (SienaStatusResponseProbeEvent event, String configName) {
        
        Enumeration deps = event.getProbeEvent ().getDeployed ();
        
        boolean deployed = false;
        while (deps.hasMoreElements () && !deployed) {
            String c = (String )deps.nextElement ();
            if (configName.equals (c)) {
                deployed = true;
            }
        }
        return deployed;
    }
    
    private boolean isInstalled (SienaStatusResponseProbeEvent event, String configName) {
        
        Enumeration ins = event.getProbeEvent ().getInstalled ();
        
        boolean installed = false;
        while (ins.hasMoreElements () && !installed) {
            String c = (String )ins.nextElement ();
            if (configName.equals (c)) {
                installed = true;
            }
        }
        return installed;
    }
    
    private boolean isActivated (SienaStatusResponseProbeEvent event, String configName) {
        
        Enumeration acts = event.getProbeEvent ().getActivated ();
        
        boolean active = false;
        while (acts.hasMoreElements () && !active) {
            String c = (String )acts.nextElement ();
            if (configName.equals (c)) {
                active = true;
            }
        }
        return active;
    }
    /** The callback invoked when a probe reports a single event.
     * @param e.  The event that was reported.
     */
    public void onProbeEvent (IProbeEvent e) {
        if (e instanceof SienaDeployResponseProbeEvent) {
            probeDeployed ();
        }
        else if (e instanceof SienaInstallResponseProbeEvent) {
            probeInstalled ();
        }
        else if (e instanceof SienaActivateResponseProbeEvent) {
            probeActivated ((SienaActivateResponseProbeEvent )e);
        }
        else if (e instanceof SienaStatusResponseProbeEvent) {
            foundProbeStatus ((SienaStatusResponseProbeEvent )e);
        }
    }
    // This is a bit of a hack - saving time
    private static final String EVENT_TYPE_KEY = "ProbeInfrastructureEventType";
    private static final String DEACTIVATE = "Deactivate";
    private static final String SYSTEM_ID_KEY = "SystemID";
    private static final String CONFIG_ID_KEY = "ProbeConfigurationName";
    private static final String UNINSTALL = "Uninstall";
    public void manageRemoval () {
        SienaProbeBus sBus = (SienaProbeBus )probeBus;       
        Notification deactivate = new Notification ();
        deactivate.putAttribute( EVENT_TYPE_KEY, DEACTIVATE );
	// was .hostname
	deactivate.putAttribute( SYSTEM_ID_KEY, systemID );
	deactivate.putAttribute( CONFIG_ID_KEY, configName);
	try{
	    sBus.siena.publish( deactivate );
	} catch ( SienaException ex ) {
	    System.err.println( "Siena error:" + ex.toString() );
	    System.out.println( "SienaException" );
	}	

        Notification uninstall = new Notification ();
        uninstall.putAttribute( EVENT_TYPE_KEY, UNINSTALL);
	// was .hostname
	uninstall.putAttribute( SYSTEM_ID_KEY, systemID);
	uninstall.putAttribute( CONFIG_ID_KEY, configName);
	
	try{
	    sBus.siena.publish( uninstall );
	} catch ( SienaException ex ) {
	    System.err.println( "Siena error:" + ex.toString() );
	    System.out.println( "SienaException" );
	}


    }
    
}
